#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "memory.h"

#define MAXWORD		128

/*
 * Binary tree example
 * Michael Plexousakis, plex@tem.uoc.gr
 * Compile with gcc -Wall -o bintree bintree.c memory.c
*/

typedef struct tnode Tnode;

struct tnode {
	char*	word;
	int	count;
	Tnode*	left;
	Tnode*	right;
};

Tnode *addtree(Tnode *p, char *w)
{
	int cond;

	if (!p) {
		p = MALLOC(sizeof (struct tnode));
		p->word = strdup(w);
		p->count = 1;
		p->left = 0;
		p->right = 0;
	}
	else {
		cond = strcmp(w, p->word);

		if (cond == 0)
			p->count++;
		else if (cond < 0)
			p->left = addtree(p->left, w);
		else
			p->right = addtree(p->right, w);
	}

	return p;
}

void treeprint(struct tnode *p)
{
	if (p) {
		treeprint(p->left);
		printf("%4d %s\n", p->count, p->word);
		treeprint(p->right);
	}
}

int main()
{
	struct tnode *root = 0;
	char word[MAXWORD], *s;
	FILE *fp;
	int j;

	fp = fopen("wotw.txt", "r");
	if (!fp) {
		printf("Could not open wotw.txt\n");
		exit(2);
	}

	while ((s = fgets(word, MAXWORD, fp))) {
		for (j = 0; *s && *s != '\n'; s++, j++);
		word[j] = '\0';

		root = addtree(root, word);
	}

	fclose(fp);

	treeprint(root);

	return 0;
}
