#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "memory.h"

void *m_malloc(int nbytes, const char *file, int line)
{
	void *ptr = 0;

	if (nbytes <= 0) {
		fprintf(stderr, "Invalid argument to MALLOC() in file %s, line %d\n", file, line);
		exit(2);
	}

	ptr = malloc(nbytes);
	if (!ptr) {
		fprintf(stderr, "MALLOC() failed in file %s, line %d\n", file, line);
		exit(2);
	}

	return ptr;
}

void *m_calloc(int count, int nbytes, const char *file, int line)
{
	void *ptr;

	if (count <= 0 || nbytes <= 0) {
		fprintf(stderr, "Invalid argument(s) to CALLOC() in file %s, line %d\n", file, line);
		exit(2);
	}

	ptr = calloc(count, nbytes);
	if (!ptr) {
		fprintf(stderr, "CALLOC() failed in file %s, line %d\n", file, line);
		exit(2);
	}

	return ptr;
}

void m_free(void *ptr, const char *file, int line)
{
	if (ptr) free(ptr);
}

void *m_realloc(void *ptr, int nbytes, const char *file, int line)
{
	void *newptr;

	if (!ptr || nbytes <= 0) {
		fprintf(stderr, "Invalid argument(s) to REALLOC() in file %s, line %d\n", file, line);
		exit(2);
	}

	newptr = realloc(ptr, nbytes);
	if (!newptr) {
		fprintf(stderr, "REALLOC() failed in file %s, line %d\n", file, line);
		exit(2);
	}

	return newptr;
}

char *m_strdup(const char *src, const char *file, int line)
{
	char *dst = malloc(strlen(src)+1);

	if (!dst) {
		fprintf(stderr, "STRUDP() failed in file %s, line %d\n", file, line);
		exit(2);
	}

	return strcpy(dst, src);
}
